#include "LsmTree_Internal.h"
#include "LsmTree.h"
#include <libxml/tree.h>
#include <libxml/parser.h>
#include "LsmTypes.h"
#include <glib.h>
#include <string.h>

	LsmTree       *
LsmTree_new(int in_withContent)
{
	LsmTree       *retval;
	xmlNodePtr      filesRoot;

	retval = g_new(LsmTree, 1);
	retval->doc=LsmTree_Internal_constructXmlDoc(in_withContent, &filesRoot);
	retval->hasContent=in_withContent;

	return retval;
}


LsmTree       *
LsmTree_newFromLsmXmlFile(char *in_lsmXmlFile)
{
	LsmTree       *retval;
	
	g_return_val_if_fail(in_lsmXmlFile, NULL);
	retval = g_new(LsmTree, 1);
	g_return_val_if_fail((retval->doc=xmlParseFile((xmlChar*)in_lsmXmlFile)), NULL);
	g_return_val_if_fail(LsmTree_Internal_isDocValid(retval), NULL);
	LsmTree_Internal_hasContentXml2Struct(retval);
	
	return retval;
}


LsmTree       *
LsmTree_newFromLsmXmlStream(char *in_lsmXmlStream)
{
	LsmTree       *retval;
	int length;

	g_return_val_if_fail(in_lsmXmlStream, NULL);
	retval = g_new(LsmTree, 1);
	length=strlen(in_lsmXmlStream);
	g_return_val_if_fail((retval->doc=xmlParseMemory((xmlChar*)in_lsmXmlStream, length)), NULL);
	g_return_val_if_fail(LsmTree_Internal_isDocValid(retval), NULL);
	LsmTree_Internal_hasContentXml2Struct(retval);

	return retval;
}


LsmTree       *
LsmTree_newFromLsmXmlDocPtr(xmlDocPtr in_lsmXmlDocPtr)
{
	LsmTree       *retval;

	retval = g_new(LsmTree, 1);
	g_return_val_if_fail(in_lsmXmlDocPtr, NULL);
	g_return_val_if_fail((retval->doc=xmlCopyDoc(in_lsmXmlDocPtr, TRUE)), NULL);
	g_return_val_if_fail(LsmTree_Internal_isDocValid(retval), NULL);
	LsmTree_Internal_hasContentXml2Struct(retval);

	return retval;
}


LsmTree       *
LsmTree_newFromLsmXmlNodePtr(xmlNodePtr in_lsmXmlNodePtr)
{
	LsmTree *retval;
	xmlNodePtr copy;

	retval = g_new(LsmTree, 1);
	g_return_val_if_fail(in_lsmXmlNodePtr, NULL);
	retval->doc = xmlNewDoc("1.0");
	copy=xmlCopyNode(in_lsmXmlNodePtr, TRUE);
	xmlDocSetRootElement(retval->doc, copy);
	g_return_val_if_fail(LsmTree_Internal_isDocValid(retval), NULL);
	LsmTree_Internal_hasContentXml2Struct(retval);

	return retval;
}


LsmTree       *
LsmTree_newFromDirectory(char *in_directory,
			  int in_withContent)
{
	LsmTree       *retval;

	g_return_val_if_fail(g_file_test(in_directory, G_FILE_TEST_IS_DIR), NULL);

	retval = g_new(LsmTree, 1);
	retval->hasContent=in_withContent;
	retval->doc=LsmTree_Internal_getXmlDocPtrFromDirectory(in_directory, in_withContent);

	return retval;
}



int
LsmTree_toLsmXmlFile(LsmTree * self,
		      char *in_lsmXmlFile)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_lsmXmlFile, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(xmlSaveFile(in_lsmXmlFile, self->doc), LSM__ERROR__MISC_ERROR);

	return retval;
}


int
LsmTree_toLsmXmlStream(LsmTree * self,
			char **out_lsmXmlStream)
{
	int             retval = 0;
	int 		size;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);

	xmlDocDumpMemory(self->doc, (xmlChar**) out_lsmXmlStream, &size);

	return retval;
}


int
LsmTree_toLsmXmlDocPtr(LsmTree * self,
			xmlDocPtr * out_lsmXmlDocPtr)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(out_lsmXmlDocPtr, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail((*out_lsmXmlDocPtr=xmlCopyDoc(self->doc, TRUE)), LSM__ERROR__NULL_POINTER);

	return retval;
}


int
LsmTree_toLsmXmlNodePtr(LsmTree * self,
			 xmlNodePtr * out_lsmXmlNodePtr)
{
	int             retval = 0;
	xmlDocPtr	copy;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail((copy=xmlCopyDoc(self->doc, TRUE)), LSM__ERROR__NULL_POINTER);
	*out_lsmXmlNodePtr=copy->children;
	xmlUnlinkNode(*out_lsmXmlNodePtr);
	xmlFreeDoc(copy);

	return retval;
}

int
LsmTree_toDirectory(LsmTree * self,
		     char *in_directory)
{
	int             retval = 0;


	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_directory, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(g_file_test(in_directory, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR), LSM__ERROR__NOT_DIR);

	LsmTree_Internal_toDirectoryRecursive(self, LsmTree_Internal_getFilesNode(self), in_directory);

	return retval;
}


int
LsmTree_destroy(LsmTree * self)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	
	if (self->doc)
		xmlFreeDoc(self->doc);

	g_free(self);

	return retval;
}
