#include <libxml/tree.h>
#include <glib.h>
#include "LsmTypes.h"
#include "LsmTree.h"
#include "LsmTree_Export.h"
#include "LsmTree_Internal.h"
#include <string.h>

 static int
 impl_LsmTree_Export_toHTMLNodeRecursive(LsmTree * self,
			   xmlNodePtr in_currentLsm,
			   xmlNodePtr in_currentHTML,
			   int deep)
{
	int             retval = 0;
	xmlNodePtr children;
	static int color=0;
	static char colors[10][7]={
		"ff0000", 	
		"ffff00",
		"000000",
		"00ff00",
		"00ffff",
		"ffffff",
		"0000ff",
		"ff00ff", 	
		"999999",
		"2299ff"
	};	

	for(children=in_currentLsm->children; children; children=children->next) {
		int type=0;
		/* 0 : nothing
		 * 1 : file
		 * 2 : dir
		 */
		
		if(!g_strcasecmp(children->name, "LsmFile")) 
			type=1;
		else  if(!g_strcasecmp(children->name, "LsmDirectory")) 
			type=2;

		if(type) {
			char *name, *auth, *user, *group, *mtime;
			xmlNodePtr node;
			xmlNodePtr tr;
			int i;

			name=xmlGetProp(children, "name");
			auth=xmlGetProp(children, "auth");
			user=xmlGetProp(children, "user");
			group=xmlGetProp(children, "group");
			mtime=xmlGetProp(children, "mtime");

			tr=xmlNewChild(in_currentHTML, NULL, "tr", NULL);
			color++;
			xmlSetProp(tr, "bgcolor", color%2?"ffffff":"aaaaaa");
			node=xmlNewChild(tr, NULL, "td", NULL);
			xmlSetProp(node, "align", "left");
			xmlNodeSetContent(node, "");
			for(i=0; i<deep; i++) {
				xmlNodePtr font;
				font=xmlNewChild(node, NULL, "font", " | ");
				xmlSetProp(font, "color", colors[i%10]);
			}
			xmlNodeAddContent(node, name);

			node=xmlNewChild(tr, NULL, "td", NULL);
			xmlSetProp(node, "align", "center");
			xmlNodeSetContent(node, auth);
			node=xmlNewChild(tr, NULL, "td", NULL);
			xmlSetProp(node, "align", "center");
			xmlNodeSetContent(node, user);
			node=xmlNewChild(tr, NULL, "td", NULL);
			xmlSetProp(node, "align", "center");
			xmlNodeSetContent(node, group);
			node=xmlNewChild(tr, NULL, "td", NULL);
			xmlSetProp(node, "align", "center");
			xmlNodeSetContent(node, mtime);
			
			if(type==1)  {
				char *size;

				size=xmlGetProp(children, "size");
				node=xmlNewChild(tr, NULL, "td", NULL);
				xmlSetProp(node, "align", "right");
				xmlNodeSetContent(node, size);
			}
			else {	// dir
				node=xmlNewChild(tr, NULL, "td", NULL);
				retval=impl_LsmTree_Export_toHTMLNodeRecursive(self, children, node, ++deep);
				g_return_val_if_fail(!retval, retval);
			}
		}
	}

	return retval;
}

 static int
 impl_LsmTree_Export_toHTMLNode(LsmTree * self,
			   xmlNodePtr* out_HTMLNode)
{
	int             retval = 0;
	xmlDocPtr htmlDoc;
	xmlNodePtr pre, node, filesNode;
	

	htmlDoc=xmlNewDoc("1.0");
	node = htmlDoc->children = xmlNewDocNode(htmlDoc, NULL, "fake", NULL);
	pre=xmlNewChild(node, NULL, "pre", NULL);
	node=xmlNewChild(pre, NULL, "table", NULL);
	xmlSetProp(node, "width", "100%");
	xmlSetProp(node, "border", "0");
	xmlSetProp(node, "cellpadding", "1");
	xmlSetProp(node, "cellspacing", "0");

	filesNode=LsmTree_Internal_getFilesNode(self);
	retval=impl_LsmTree_Export_toHTMLNodeRecursive(self, filesNode, node, 0);
	g_return_val_if_fail(!retval, retval);
	
	xmlUnlinkNode(pre);
	xmlFreeDoc(htmlDoc);
	*out_HTMLNode=pre;
	
	return retval;
}

static int
 impl_LsmTree_Export_toHTMLDoc(LsmTree * self,
			   xmlDocPtr* out_HTMLDoc)
{
	int             retval = 0;
	xmlDocPtr htmlDoc;
	xmlNodePtr node, node2;

	htmlDoc=xmlNewDoc("1.0");
	node = htmlDoc->children = xmlNewDocNode(htmlDoc, NULL, "html", NULL);
	node=xmlNewChild(node, NULL, "head", NULL);
	node=xmlNewChild(node, NULL, "title", "LsmLib Doc Exported into HTML");
	node = htmlDoc->children;
	node=xmlNewChild(node, NULL, "body", NULL);
	retval= impl_LsmTree_Export_toHTMLNode(self, &node2);
	g_return_val_if_fail(!retval, retval);
	xmlAddChild(node, node2);
	
	*out_HTMLDoc=htmlDoc;

	return retval;
}

static int
impl_LsmTree_Export_toHTML(LsmTree * self,
		       char **out_htmlStream)
{
	int             retval = 0;
	xmlDocPtr htmlDoc;
	xmlNodePtr node;
	int size;

	htmlDoc=xmlNewDoc("1.0");
	retval= impl_LsmTree_Export_toHTMLNode(self, &node);
	g_return_val_if_fail(!retval, retval);
	xmlDocSetRootElement(htmlDoc, node);
	xmlDocDumpMemory(htmlDoc, (xmlChar**)out_htmlStream, &size);
	g_return_val_if_fail(size, LSM__ERROR__NO_MEMORY);
	xmlFreeDoc(htmlDoc);

	return retval;
}

 static int
 impl_LsmTree_Export_toHTMLFile(LsmTree * self,
			   char *in_HTMLFile)
{
	int             retval = 0, x;
	xmlDocPtr htmlDoc;
	retval= impl_LsmTree_Export_toHTMLDoc(self, &htmlDoc);
	g_return_val_if_fail(!retval, retval);
	x=xmlSaveFile(in_HTMLFile, htmlDoc);
	g_return_val_if_fail(x, LSM__ERROR__INVALID_FILENAME);

	return retval;
}

static int
impl_LsmTree_Export_toStringRecursive(LsmTree * self,
			   xmlNodePtr in_currentLsm,
			   GList** out_stringList,
			   int deep)
{
	int             retval = 0;
	xmlNodePtr children;

	for(children=in_currentLsm->children; children; children=children->next) {
		int type=0;
		/* 0 : nothing
		 * 1 : file
		 * 2 : dir
		 */
		
		if(!g_strcasecmp(children->name, "LsmFile")) 
			type=1;
		else  if(!g_strcasecmp(children->name, "LsmDirectory")) 
			type=2;

		if(type) {
			static int symbol=0;
			char *name, *mtime, *fill1, *fill2;
			char *new;
			int nameLength;
			int mtimeLength;
			int fill1Length;
			int fill2Length;

			name=xmlGetProp(children, "name");
			mtime=xmlGetProp(children, "mtime");
			nameLength=strlen(name);
			mtimeLength=strlen(mtime);
			symbol++;
			if(deep>13) {
				fill1=g_strdup((symbol%2)?"----------...":"++++++++++...");
				fill1Length=13;
			}
			else {
				fill1=g_strnfill(deep, (symbol%2)?'-':'+');
				fill1Length=deep;
			}
			fill2Length=70-fill1Length-nameLength-mtimeLength;
			if (fill2Length<1)
				fill2=g_strdup("\t");
			else
				fill2=g_strnfill(fill2Length, ' ');

			if(type==1)  {
				char *size;
				int sizeLength;
				char *fill3;
				int fill3Length;

				size=xmlGetProp(children, "size");
				sizeLength=strlen(size);
				fill3Length=20-sizeLength;
				if (fill3Length<1)
					fill3=g_strdup("\t");
				else
					fill3=g_strnfill(fill3Length, ' ');
				new=g_strconcat(" ", fill1, name, fill2, mtime, fill3, size, NULL);
				
				g_free(fill3);
			}
			else
				new=g_strconcat("d", fill1, name, fill2, mtime, NULL);

			g_free(fill1);
			g_free(fill2);

			*out_stringList=g_list_append(*out_stringList, new);
			if(type==2) {
				retval=impl_LsmTree_Export_toStringRecursive(self, children, out_stringList, ++deep);
				g_return_val_if_fail(!retval, retval);
			}

		}
	}

	return retval;
}


 static int
 impl_LsmTree_Export_toString(LsmTree * self,
			 char **out_string)
{
	int             retval = 0;
	xmlNodePtr filesNode;
	GList *stringList=NULL, *stringListC=NULL;
	int stringSize=0;
	char *string;
	int i=0;

	filesNode=LsmTree_Internal_getFilesNode(self);
	retval=impl_LsmTree_Export_toStringRecursive(self, filesNode, &stringList, 0);
	g_return_val_if_fail(!retval, retval);

	for(stringListC=stringList; stringListC; stringListC=g_list_next(stringListC)) {
		char* c;

		c=(char*)(stringListC->data);
		stringSize+=strlen(c);
		stringSize++;	// '\n'
	}
	stringSize++;	// '\0'

	string=g_new(char, stringSize);
	for(stringListC=stringList; stringListC; stringListC=g_list_next(stringListC)) {
		int j;
		char* c;

		c=(char*)(stringListC->data);

		for(j=0; c[j]; j++) {
			string[i]=c[j];
			i++;
		}
		string[i]='\n';
		i++;
		g_free(stringListC->data);
	}
		
	string[i]='\0';
	*out_string=string;
	g_list_free(stringList);
	
	return retval;
}


 static int
 impl_LsmTree_Export_toStringOnStdout(LsmTree * self)
{
	int             retval = 0;
	char* string;

	retval=impl_LsmTree_Export_toString(self, &string);
	g_return_val_if_fail(!retval, retval);

	printf(string);

	g_free(string);

	return retval;
}

int
LsmTree_Export_toHTML(LsmTree * self,
		       char **out_htmlStream)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(out_htmlStream, LSM__ERROR__NULL_POINTER);

	retval=impl_LsmTree_Export_toHTML(self, out_htmlStream);
	g_return_val_if_fail(!retval, retval);

	return retval;
}

int
LsmTree_Export_toHTMLFile(LsmTree * self,
			   char *in_HTMLFile)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_HTMLFile, LSM__ERROR__NULL_POINTER);

	retval=impl_LsmTree_Export_toHTMLFile(self, in_HTMLFile);
	g_return_val_if_fail(!retval, retval);

	return retval;
}


int
LsmTree_Export_toHTMLNode(LsmTree * self,
			   xmlNodePtr* out_HTMLNode)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(out_HTMLNode, LSM__ERROR__NULL_POINTER);

	retval=impl_LsmTree_Export_toHTMLNode(self, out_HTMLNode);
	g_return_val_if_fail(!retval, retval);

	return retval;
}

int
LsmTree_Export_toString(LsmTree * self,
			 char **out_string)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(out_string, LSM__ERROR__NULL_POINTER);

	retval=impl_LsmTree_Export_toString(self, out_string);
	g_return_val_if_fail(!retval, retval);

	return retval;
}


int
LsmTree_Export_toStringOnStdout(LsmTree * self)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);

	retval=impl_LsmTree_Export_toStringOnStdout(self);
	g_return_val_if_fail(!retval, retval);

	return retval;
}

