#include "config.h"
#include <glib.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include "LsmTypes.h"
#include "LsmTree.h"
#include "LsmBase64.h"
#include "LsmTree_Internal.h"
#include <sys/stat.h>	
#include <sys/types.h>		
#include <stdio.h>		
#include <stdio.h>
#include <sys/types.h>		/* for opendir , stat, getpwent, endpwent */
#include <dirent.h>		/* for opendir */
#include <stdlib.h>
#include <string.h>		/* for strlen, strcpy, ... */
#include <sys/stat.h>		/* for stat */
#include <unistd.h>		/* for stat */
#include <pwd.h>		/* to handle with users */
#include <grp.h>		/* to handle with groups */
#include <time.h>		/* date of modif. ... */
#include <errno.h>

#ifdef H_UseMD5
#include <md5.h>
#endif

#include "LsmTypes.h"
#include "LsmBase64.h"
#include "LsmTree_Internal.h"

#include <math.h>	// pow

int
LsmTree_Internal_addPath(const char *tree,
		      const char *flower,
		      char fullName[LSM_MAXPATHSIZE])
{
	if ((strlen(tree) + strlen(flower) + 2) > LSM_MAXPATHSIZE) {
		fprintf(stderr, "Error LsmTree_Internal_addPath : path too long");
		return (1);
	}
	strcpy(fullName, tree);
	strcat(fullName, G_DIR_SEPARATOR_S);
	strcat(fullName, flower);	

	return (0);
}

 int
LsmTree_Internal_mode_t2string(mode_t umt,
			    char name[LSM_MAXAUTHSIZE])
{
	name[0] = (S_IRUSR & umt) ? 'r' : '-';
	name[1] = (S_IWUSR & umt) ? 'w' : '-';
	name[2] = (S_IXUSR & umt) ? 'x' : '-';
	name[3] = (S_IRGRP & umt) ? 'r' : '-';
	name[4] = (S_IWGRP & umt) ? 'w' : '-';
	name[5] = (S_IXGRP & umt) ? 'x' : '-';
	name[6] = (S_IROTH & umt) ? 'r' : '-';
	name[7] = (S_IWOTH & umt) ? 'w' : '-';
	name[8] = (S_IXUSR & umt) ? 'x' : '-';
	name[9] = '\0';

	return (0);
}


 int
LsmTree_Internal_time_t2string(time_t td,
			    char name[LSM_MAXTIMESIZE])
{
	/* we need to delete the '\n' character */
	strcpy(name, ctime(&td));
	name[24]='\0';

	return (0);
}

 int
LsmTree_Internal_int2string(int size,
			 char name[LSM_MAXSIZEOFSIZE])
{
	if (sprintf(name, "%d", size) == -1) {

		fprintf(stderr, "Error sprintf, value may be trunked");
		return (1);
	}
	return (0);
}

 int
LsmTree_Internal_off_t2string(off_t size,
			   char name[LSM_MAXSIZEOFSIZE])
{
	if (LsmTree_Internal_int2string((int)size, name) != 0) {
		fprintf(stderr, "Error LsmTree_Internal_int2string");
		return (1);
	}
	return (0);
}

 int
LsmTree_Internal_uid2string(uid_t uid,
			 char name[LSM_MAXUSERNAMESIZE])
{
	struct passwd  *myPwEnt;
	myPwEnt = getpwuid(uid);
	if (strlen(myPwEnt->pw_name) > LSM_MAXUSERNAMESIZE) {
		fprintf(stderr, "Error strcpy, user name too long");
		return (1);
	}
	strcpy(name, myPwEnt->pw_name);
	return (0);
}

 int
LsmTree_Internal_gid2string(gid_t gid,
			 char name[LSM_MAXGROUPNAMESIZE])
{
	struct group   *myGrEnt;
	myGrEnt = getgrgid(gid);
	if (strlen(myGrEnt->gr_name) > LSM_MAXGROUPNAMESIZE) {
		fprintf(stderr, "Error strcpy, group name too long");
		return (1);
	}
	strcpy(name, myGrEnt->gr_name);
	return (0);
}


/*
 * ********** LsmTree_Internal_scanDir **********************
 * 
 * LsmTree_Internal_scanDir analyse a directory and complete the associated
 * dirNode. It is a recursive function. ****
 */

 int
LsmTree_Internal_scanDir(const char *dir,
		      xmlNodePtr dirNode,
			  int in_withContent)
{
	DIR            *myDir;
	struct dirent  *myDirEnt;
	struct stat     myStat;
	xmlNodePtr      myTreeElement;
	char            fullName[LSM_MAXPATHSIZE];
	char            userName[LSM_MAXUSERNAMESIZE];
	char            groupName[LSM_MAXGROUPNAMESIZE];
	char            sizeName[LSM_MAXSIZEOFSIZE];
	char            atimeName[LSM_MAXTIMESIZE];	/* access */
	char            mtimeName[LSM_MAXTIMESIZE];	/* modified */
	char            ctimeName[LSM_MAXTIMESIZE];	/* creation */
	char            authName[LSM_MAXAUTHSIZE];	/* authorisation /
							 * permission */

	g_return_val_if_fail(((myDir = opendir(dir)) != NULL), LSM__ERROR__OPEN_DIR);

	while ((myDirEnt = readdir(myDir)) != NULL) {
		LsmTree_Internal_addPath(dir, myDirEnt->d_name, fullName);

		g_return_val_if_fail(!stat(fullName, &myStat), LSM__ERROR__STAT);
		
		if (S_ISDIR(myStat.st_mode)) {
			if (strcmp(myDirEnt->d_name, ".") && strcmp(myDirEnt->d_name, "..")) {
				myTreeElement = xmlNewChild(dirNode, NULL, "LsmDirectory", NULL);
				/* Recursive call of function */
				if (LsmTree_Internal_scanDir(fullName, myTreeElement, in_withContent) != 0) {
					/*
					 * Nothing to do in particulary It is
					 * the way of life that sometimes you
					 * can not stat a dir element if you
					 * do not have permission in the
					 * current directory ****
					 */
					g_warning("Warning scandir, scandir %s failed\n", fullName);
				}
			}
			else 
				myTreeElement = NULL;

		} 
		else { 
			myTreeElement = xmlNewChild(dirNode, NULL, "LsmFile", NULL);
			g_return_val_if_fail(!LsmTree_Internal_off_t2string(myStat.st_size, sizeName), LSM__ERROR__MISC_ERROR);
			xmlSetProp(myTreeElement, "size", sizeName);
			if(in_withContent) {
				int destSize;
				char* content;

				LsmBase64_encodeFromFile(fullName, &content, &destSize);
				xmlNodeSetContent(myTreeElement, content);
			}
#ifdef H_UseMD5
			else {
				char md5[33];

				MD5File(fullName, md5);
				xmlSetProp(myTreeElement, "md5", md5);
			}
#endif
		}

		if(myTreeElement) {
			xmlSetProp(myTreeElement, "name", myDirEnt->d_name);
			g_return_val_if_fail(!LsmTree_Internal_uid2string(myStat.st_uid, userName), LSM__ERROR__MISC_ERROR); 
			xmlSetProp(myTreeElement, "user", userName);
			g_return_val_if_fail(!LsmTree_Internal_gid2string(myStat.st_gid, groupName), LSM__ERROR__MISC_ERROR); 
			xmlSetProp(myTreeElement, "group", groupName);
			g_return_val_if_fail(!LsmTree_Internal_time_t2string(myStat.st_atime, atimeName), LSM__ERROR__MISC_ERROR);
			xmlSetProp(myTreeElement, "atime", atimeName);
			g_return_val_if_fail(!LsmTree_Internal_time_t2string(myStat.st_mtime, mtimeName), LSM__ERROR__MISC_ERROR);
			xmlSetProp(myTreeElement, "mtime", mtimeName);
			g_return_val_if_fail(!LsmTree_Internal_time_t2string(myStat.st_ctime, ctimeName), LSM__ERROR__MISC_ERROR);
			xmlSetProp(myTreeElement, "ctime", ctimeName);
			g_return_val_if_fail(!LsmTree_Internal_mode_t2string(myStat.st_mode, authName), LSM__ERROR__MISC_ERROR);
			xmlSetProp(myTreeElement, "auth", authName);
		}
	}
	closedir(myDir);
	return (0);
}

 xmlDocPtr
LsmTree_Internal_constructXmlDoc(int in_withContent, 
		xmlNodePtr* out_filesRoot)
{
	xmlDocPtr       retval;
	xmlNodePtr      filesRoot;
	xmlNodePtr	comment;

	retval = xmlNewDoc("1.0");
	retval->children = xmlNewDocNode(retval, NULL, "LsmTree", NULL);
	comment=xmlNewComment(LSM_COPYRIGHT);
	xmlAddChild(retval->children, comment);
	comment=xmlNewComment(LSM_GENERATED);
	xmlAddChild(retval->children, comment);
	xmlSetProp(retval->children, "hasContent", in_withContent?"yes":"no");
	xmlSetProp(retval->children, "VFS_defaultAuth", "rw-r--r--");
	xmlSetProp(retval->children, "VFS_defaultOwner", "root");
	xmlSetProp(retval->children, "VFS_defaultGroup", "root");

	filesRoot = xmlNewChild(retval->children, NULL, "LsmFiles", NULL);
	*out_filesRoot=filesRoot;

	return retval;
}

 xmlDocPtr
LsmTree_Internal_getXmlDocPtrFromDirectory(const char *dir,
			  int in_withContent)
{
	xmlDocPtr       retval;
	xmlNodePtr      filesRoot;

	retval = LsmTree_Internal_constructXmlDoc(in_withContent, &filesRoot);
		
	g_return_val_if_fail(!LsmTree_Internal_scanDir(dir, filesRoot, in_withContent), NULL);
	
	return retval;
}


 void
LsmTree_Internal_hasContentXml2Struct(LsmTree * self)
{
	char* hasContentS;
	int hasContent;

	hasContentS=xmlGetProp(self->doc->children, "hasContent");
	hasContent=!g_strcasecmp(hasContentS, "yes");
	self->hasContent=hasContent;
}

 int
LsmTree_Internal_isDocValid(LsmTree * self)
{
	int retval=1;
	xmlNodePtr node;

	// FIXME: One day, we should use an xml schema!
	
	g_return_val_if_fail(self, 0);
	g_return_val_if_fail(self->doc, 0);
	g_return_val_if_fail(self->doc->children, 0);
	node=self->doc->children;
	g_return_val_if_fail(!g_strcasecmp(node->name, "LsmTree"), 0);

	return retval;
}

static int
impl_LsmTree_Internal_string2mode_t(char* in_string,
			mode_t* out_mt)
{
	int i;
	int power=1;
	int retval=0;

	*out_mt=0;

	for(i=0; i<=8; i++) {
		if(in_string[8-i]!='-')
			*out_mt+=power;
		power*=2;
	}

	return retval;
}

int
LsmTree_Internal_saveFileAtFromXmlNodePtr(LsmTree* self,
				xmlNodePtr fileNode,
				char *at)
{
	int retval=0;

	g_return_val_if_fail(!g_strcasecmp(fileNode->name, "LsmFile"), LSM__ERROR__NOT_REGULAR_FILE);

	if(self->hasContent) {
		char* content;

		content=xmlNodeGetContent(fileNode);
		LsmBase64_decodeToFile(content, at);
	}
	else {
		FILE* fd;
		char* md5;

		md5=xmlGetProp(fileNode, "md5");

		fd=fopen(at, "w");
		fprintf(fd, "md5=%s", md5);
		fclose(fd);
	}

	return retval;
}

void
LsmTree_Internal_toDirectoryRecursive(LsmTree * self,
			xmlNodePtr current,
		     char *in_directory)
{
	xmlNodePtr children;

	children=current->children;

	while(children) {
		int type=0;
		/* 0 : nothing
		 * 1 : file
		 * 2 : dir
		 */
		
		if(!g_strcasecmp(children->name, "LsmFile")) 
			type=1;
		else  if(!g_strcasecmp(children->name, "LsmDirectory")) 
			type=2;

		if(type) {
			char* name;
			char* authS;
			mode_t auth;
			char* fullName;

			name=xmlGetProp(children, "name");
			authS=xmlGetProp(children, "auth");
			impl_LsmTree_Internal_string2mode_t(authS, &auth);
			fullName=g_build_filename(in_directory, name, NULL);

			if(type==1)  // file
				LsmTree_Internal_saveFileAtFromXmlNodePtr(self, children, fullName);
			else {	// dir
				mkdir(fullName, S_IRWXU); // need all rights for continuing

				LsmTree_Internal_toDirectoryRecursive(self, children, fullName);
			}
				
			chmod(fullName, auth);
			g_free(fullName);
		}
		children=children->next;
	}
}

xmlNodePtr
LsmTree_Internal_getFilesNode(LsmTree * self)
{
	xmlNodePtr retval=NULL;
	xmlNodePtr lsmtree;
	xmlNodePtr lsmtreeChildren;

	g_return_val_if_fail(self, NULL);
	g_return_val_if_fail(self->doc, NULL);
	g_return_val_if_fail((lsmtree=self->doc->children), NULL);
	g_return_val_if_fail((lsmtreeChildren=lsmtree->children), NULL);

	while(lsmtreeChildren) {
		if(!g_strcasecmp(lsmtreeChildren->name, "LsmFiles")) {
			retval=lsmtreeChildren;
			break;
		}
		lsmtreeChildren=lsmtreeChildren->next;
	}
	g_return_val_if_fail(retval, NULL);
	
	return retval;
}


