#include "config.h"
#include "LsmTypes.h"
#include "LsmTree.h"
#include "LsmTree_Internal.h"
#include "LsmTree_VFS.h"
#include "LsmBase64.h"
#include <libxml/tree.h>
#include <glib.h>
#ifdef H_UseMD5
#  include <md5.h>
#endif
#include <string.h>
#include <time.h>

void
printSplitted(char** splitted) 
{
	int nb=0;

	while(splitted[nb]) {
		printf("%d: %s\n", nb, splitted[nb]);
		nb++;
	}
}

static int 
impl_LsmTree_VFS_getXmlNodePtrFromPath(LsmTree* self,
		char* in_path,
		xmlNodePtr* out_node)
{
	int retval=0;
	xmlNodePtr filesNode;
	char** splitted;

	*out_node=NULL;

	filesNode=LsmTree_Internal_getFilesNode(self);
	splitted=g_strsplit(in_path, "/", 0);

	//printSplitted(splitted);
	
	if(splitted[0] &&  !splitted[0][0] && splitted[1] && !splitted[1][0] && !splitted[2])
		*out_node=filesNode;
	else {
		int i=1;
		int found=0;
		xmlNodePtr current;

		current=filesNode->children;

		for(i=1; splitted[i]; i++) {

			//printf("splitted[i]=%s\n", splitted[i]);
			found=0;
			//printf("current : %d\n", (int) current);
			while(current){
				//printf("current : %d\n", (int) current);
				if(!g_strcasecmp(current->name, "LsmFile") || !g_strcasecmp(current->name, "LsmDirectory")) {
					char* name;

					name=xmlGetProp(current, "name");
					if(!g_strcasecmp(name, splitted[i])) {
						found=1;
						break;
					}
				}
				current=current->next;
			}
			g_return_val_if_fail(found, LSM__ERROR__INVALID_PATH);
			if(splitted[i+1])
				current=current->children;
		}
		//printf("current : %d\n", (int) current);
		*out_node=current;
	}
	
	g_strfreev(splitted);
	g_return_val_if_fail(*out_node, LSM__ERROR__MISC_ERROR);

	return retval;
}

static int
impl_LsmTree_VFS_saveFileAt(LsmTree * self,
			char *in_file,
			char *in_at)
{
	int             retval = 0;
	xmlNodePtr	node;


	retval=impl_LsmTree_VFS_getXmlNodePtrFromPath(self, in_file, &node);
	g_return_val_if_fail(!retval, retval);
	retval=LsmTree_Internal_saveFileAtFromXmlNodePtr(self, node, in_at);
	g_return_val_if_fail(!retval, retval);

	return retval;
}

int
LsmTree_VFS_saveFileAt(LsmTree * self,
			char *in_file,
			char *in_at)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_file, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_at, LSM__ERROR__NULL_POINTER);

	retval=impl_LsmTree_VFS_saveFileAt(self, in_file, in_at);
	
	return retval;
}

static int
impl_LsmTree_VFS_saveDirectoryAt(LsmTree * self,
			     char *in_directory,
			     char *in_at)
{
	int retval=0;
	xmlNodePtr	node;

	impl_LsmTree_VFS_getXmlNodePtrFromPath(self, in_directory, &node);
	g_return_val_if_fail(node, LSM__ERROR__INVALID_PATH);
	LsmTree_Internal_toDirectoryRecursive(self, node, in_at);

	return retval;
}

int
LsmTree_VFS_saveDirectoryAt(LsmTree * self,
			     char *in_directory,
			     char *in_at)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_directory, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_at, LSM__ERROR__NULL_POINTER);

	retval=impl_LsmTree_VFS_saveDirectoryAt(self, in_directory, in_at);

	return retval;
}

static int
impl_LsmTree_VFS_getFileAtAsStream(LsmTree * self,
			       char *in_at,
			       char **out_stream,
			       int *out_length)
{
	int retval=0;
	xmlNodePtr node;

	retval=impl_LsmTree_VFS_getXmlNodePtrFromPath(self, in_at, &node);
	g_return_val_if_fail(!retval, retval);

	//printf("node is %d\n", (int)node);
	if(self->hasContent) {
		char* content;

		content=xmlNodeGetContent(node);
		retval=LsmBase64_decodeToStream(content, out_stream, out_length);
	}
	else {
		char* md5;

		md5=xmlGetProp(node, "md5");

		printf("md5 is %s\n", md5);
		printf("name is %s\n", xmlGetProp(node, "name"));

		if(out_length)
			*out_length=37; // strlen("md5=") + 32

		*out_stream=g_strdup_printf("md5=%s", md5);
	}

	return retval;
}

int
LsmTree_VFS_getFileAtAsStream(LsmTree * self,
			       char *in_at,
			       char **out_stream,
			       int *out_length)
{
	int retval=0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(out_stream, LSM__ERROR__NULL_POINTER);

	retval=impl_LsmTree_VFS_getFileAtAsStream(self, in_at, out_stream, out_length);
	
	return retval;
}

static int
impl_LsmTree_VFS_setAllTimesToCurrentOnNode(LsmTree * self,
		xmlNodePtr node)
{
	int retval=0;
	time_t tt;
	char tts[26];

	tt=time(NULL);
	ctime_r(&tt, tts);
	tts[24]='\0';

	xmlSetProp(node, "atime", tts);
	xmlSetProp(node, "mtime", tts);
	xmlSetProp(node, "ctime", tts);

	return retval;
}

static int
impl_LsmTree_VFS_newItem(LsmTree * self,
			       xmlNodePtr motherNode,
			       char *in_type,
			       char *in_name,
			       char *in_auth,
			       char *in_owner,
			       char *in_group,
			       xmlNodePtr* out_newItem)
{
	int retval = 0;
	xmlNodePtr node;

	node=xmlNewChild(motherNode, NULL, in_type, NULL);
	xmlSetProp(node, "name", (xmlChar*)in_name);
	xmlSetProp(node, "auth", in_auth?(xmlChar*)in_auth:xmlGetProp(self->doc->children, "VFS_defaultAuth"));
	xmlSetProp(node, "user", in_owner?(xmlChar*)in_owner:xmlGetProp(self->doc->children, "VFS_defaultOwner"));
	xmlSetProp(node, "group", in_group?(xmlChar*)in_group:xmlGetProp(self->doc->children, "VFS_defaultGroup"));

	retval=impl_LsmTree_VFS_setAllTimesToCurrentOnNode(self, node);
	g_return_val_if_fail(!retval, retval);

	*out_newItem=node;
	
	return retval;
}

static int
impl_LsmTree_VFS_newFile(LsmTree * self,
			       char *in_stream,
			       int in_streamLength,
			       xmlNodePtr motherNode,
			       char *in_name,
			       char *in_auth,
			       char *in_owner,
			       char *in_group)
{
	int retval=0;
	xmlNodePtr node;

	retval=impl_LsmTree_VFS_newItem(self, motherNode, "LsmFile", in_name, in_auth, in_owner, in_group, &node);
	g_return_val_if_fail(!retval, retval);
	
	if(self->hasContent) {
		char *content;

		retval=LsmBase64_encodeFromStream(in_stream, in_streamLength, &content, NULL);
		g_return_val_if_fail(!retval, retval);

		xmlNodeSetContent(node, content);
	}
#ifdef H_UseMD5
	else {
		MD5_CTX context;
		char md5[33];

		MD5Init(&context);
		MD5Update(&context, in_stream, in_streamLength?in_streamLength:strlen(in_stream));
		MD5End(&context, md5);

		xmlSetProp(node, "md5", md5);
	}
#endif
	
	return retval;
}


static int
impl_LsmTree_VFS_newDirectory(LsmTree * self,
			       xmlNodePtr motherNode,
			       char *in_name,
			       char *in_auth,
			       char *in_owner,
			       char *in_group,
			       xmlNodePtr* out_newDirectory)
{
	int             retval = 0;

	retval=impl_LsmTree_VFS_newItem(self, motherNode, "LsmDirectory", in_name, in_auth, in_owner, 
			in_group, out_newDirectory);
	g_return_val_if_fail(!retval, retval);
	
	return retval;
}

static int
impl_LsmTree_VFS_getMotherNodeFromPathCreatingDirectoryIfNecessary(LsmTree* self,
				char* in_path,
				xmlNodePtr* out_motherNode,
				char *in_auth,
			       char *in_owner,
			       char *in_group)
{
	int retval=0;
	xmlNodePtr filesNode;
	gchar** splitted;

	*out_motherNode=NULL;

	filesNode=LsmTree_Internal_getFilesNode(self);
	splitted=g_strsplit(in_path, "/", 0);

	//printSplitted(splitted);
	
	if(splitted[0] &&  !splitted[0][0] && splitted[1] && !splitted[1][0] && !splitted[2])
		*out_motherNode=filesNode;
	else {
		int i=1;
		int found=0;
		xmlNodePtr current;
		xmlNodePtr parent;

		current=filesNode->children;
		parent=filesNode;

		for(i=1; splitted[i+1]; i++) {
			
			found=0;
			while(current){
				if(!g_strcasecmp(current->name, "LsmDirectory")) {
					char* name;

					name=xmlGetProp(current, "name");
					if(!g_strcasecmp(name, splitted[i])) {
						found=1;
						break;
					}
				}
				current=current->next;
			}
			if(!found) {
				retval=impl_LsmTree_VFS_newDirectory(self, parent, splitted[i], in_auth, in_owner, 
						in_group, &current);
				g_return_val_if_fail(!retval, retval);
			}
			//else {
			//	printf("found %s\n", splitted[i]);
			//}
		
			parent=current;
			current=current->children;
		}
		*out_motherNode=parent;
	}
	
	g_strfreev(splitted);

	g_return_val_if_fail(*out_motherNode, LSM__ERROR__MISC_ERROR);

	return retval;
}


static int
impl_LsmTree_VFS_getBaseName(LsmTree * self,
				char *in_path,
				char **out_basename)
{
	int retval=0;
	char** splitted;
	int i;
	
	splitted=g_strsplit(in_path, "/", 0);
	g_return_val_if_fail(splitted && splitted[0], LSM__ERROR__MISC_ERROR);

	for(i=0; splitted[i+1]; i++)
		;

	*out_basename=g_strdup(splitted[i]);
	
	g_strfreev(splitted);

	return retval;
}
	
static int
impl_LsmTree_VFS_addStreamAsFileAt(LsmTree * self,
			       char *in_stream,
			       int in_streamLength,
			       char *in_at,
			       char *in_auth,
			       char *in_owner,
			       char *in_group)
{
	int retval=0;
	xmlNodePtr motherNode;
	char *basename;

	retval=impl_LsmTree_VFS_getMotherNodeFromPathCreatingDirectoryIfNecessary(self, in_at, &motherNode, 
			in_auth, in_owner, in_group);
	g_return_val_if_fail(!retval, retval);
	retval=impl_LsmTree_VFS_getBaseName(self, in_at, &basename);
	g_return_val_if_fail(!retval, retval);
	retval=impl_LsmTree_VFS_newFile(self, in_stream, in_streamLength, motherNode, basename,
			       in_auth, in_owner, in_group);
	g_return_val_if_fail(!retval, retval);
	g_free(basename);
	
	return retval;
}

int
LsmTree_VFS_addStreamAsFileAt(LsmTree * self,
			       char *in_stream,
			       int in_streamLength,
			       char *in_at,
			       char *in_auth,
			       char *in_owner,
			       char *in_group)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_stream, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_at, LSM__ERROR__NULL_POINTER);

	retval=impl_LsmTree_VFS_addStreamAsFileAt(self, in_stream, in_streamLength, in_at, in_auth, in_owner, in_group);
	
	return retval;
}

static int
impl_LsmTree_VFS_addFileAt(LsmTree * self,
		       char *in_file,
		       char *in_at,
		       char *in_auth,
		       char *in_owner,
		       char *in_group)
{
	int             retval = 0;
	char* stream;
	int streamLength;
	GError* error=NULL;

	g_return_val_if_fail(g_file_get_contents(in_file, &stream, &streamLength, &error), LSM__ERROR__FILE_NOT_FOUND);
	
	retval=impl_LsmTree_VFS_addStreamAsFileAt(self, stream, streamLength, in_at, in_auth, in_owner, in_group);
	g_free(stream);

	return retval;
}

int
LsmTree_VFS_addFileAt(LsmTree * self,
		       char *in_file,
		       char *in_at,
		       char *in_auth,
		       char *in_owner,
		       char *in_group)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_file, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_at, LSM__ERROR__NULL_POINTER);

	retval=impl_LsmTree_VFS_addFileAt(self, in_file, in_at, in_auth, in_owner, in_group);

	return retval;
}

static int
impl_LsmTree_VFS_addDirectoryAt(LsmTree * self,
			    char *in_directory,
			    char *in_at)
{
	int             retval = 0;
	xmlNodePtr motherNode;
	xmlNodePtr node;
	char *basename;

	retval=impl_LsmTree_VFS_getMotherNodeFromPathCreatingDirectoryIfNecessary(self, in_at, &motherNode, 
			NULL, NULL, NULL);
	g_return_val_if_fail(!retval, retval);
	retval=impl_LsmTree_VFS_getBaseName(self, in_at, &basename);
	g_return_val_if_fail(!retval, retval);
	retval=impl_LsmTree_VFS_newDirectory(self, motherNode, basename,
			NULL, NULL, NULL, &node);
	g_return_val_if_fail(!retval, retval);

	LsmTree_Internal_scanDir(in_directory, node, self->hasContent);
	
	return retval;
}

int
LsmTree_VFS_addDirectoryAt(LsmTree * self,
			    char *in_directory,
			    char *in_at)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_directory, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_at, LSM__ERROR__NULL_POINTER);

	retval=impl_LsmTree_VFS_addDirectoryAt(self, in_directory, in_at);
	
	return retval;
}

int
LsmTree_VFS_insertAnotherLsmTreeAt(LsmTree * self,
				    LsmTree * anotherLsmTree,
				    char *in_at)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(anotherLsmTree, LSM__ERROR__NULL_POINTER);

	return retval;
}

int
LsmTree_VFS_setDefaultAuth(LsmTree * self,
			    char *in_defaultAuth)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_defaultAuth, LSM__ERROR__NULL_POINTER);

	xmlSetProp(self->doc->children, "VFS_defaultAuth", in_defaultAuth);
	
	return retval;
}

int
LsmTree_VFS_getDefaultAuth(LsmTree * self,
			    char **out_defaultAuth)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(out_defaultAuth, LSM__ERROR__NULL_POINTER);

	*out_defaultAuth=xmlGetProp(self->doc->children, "VFS_defaultAuth");
	
	return retval;
}

int
LsmTree_VFS_setDefaultOwner(LsmTree * self,
			     char *in_defaultOwner)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_defaultOwner, LSM__ERROR__NULL_POINTER);

	xmlSetProp(self->doc->children, "VFS_defaultOwner", in_defaultOwner);

	return retval;
}

int
LsmTree_VFS_getDefaultOwner(LsmTree * self,
			     char **out_defaultOwner)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(out_defaultOwner, LSM__ERROR__NULL_POINTER);

	*out_defaultOwner=xmlGetProp(self->doc->children, "VFS_defaultOwner");
	
	return retval;
}

int
LsmTree_VFS_setDefaultGroup(LsmTree * self,
			     char *in_defaultGroup)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(in_defaultGroup, LSM__ERROR__NULL_POINTER);

	xmlSetProp(self->doc->children, "VFS_defaultGroup", in_defaultGroup);
	
	return retval;
}

int
LsmTree_VFS_getDefaultGroup(LsmTree * self,
			     char **out_defaultGroup)
{
	int             retval = 0;

	g_return_val_if_fail(self, LSM__ERROR__NULL_POINTER);
	g_return_val_if_fail(out_defaultGroup, LSM__ERROR__NULL_POINTER);

	*out_defaultGroup=xmlGetProp(self->doc->children, "VFS_defaultGroup");
	
	return retval;
}


