#include <lsmlib.h>
#include <glib.h>
#include "main.h"
#include <stdlib.h> /* exit */
#include <stdio.h>
#include <unistd.h> /* for getopt */

static LsmOpts *
LsmOpts_new(int argc, char **argv)
{
	LsmOpts        *retval;
	int             optch;
	static char     optstring[] = "i:o:hv";

	retval = g_new(LsmOpts, 1);

	/* setting defaults */
	retval->inputFilter=LSM__INPUT_FILTER__DIR;
	retval->outputFilter=LSM__OUTPUT_FILTER__XML_FILE;
	retval->input=NULL;
	retval->output=NULL;

	/* processing options */
	while ((optch = getopt(argc, argv, optstring)) != -1) {
		switch (optch) {

		case 'h':
			printf("lsm2, version %s \n\n %s", VERSION, USAGE);
			exit(0);
			break;

		case 'v':
			printf("lsm2, version %s\n", VERSION);
			exit (0);
			break;

		case 'i':
			if(!g_strcasecmp(optarg, "xmlFile"))
				retval->inputFilter=LSM__INPUT_FILTER__XML_FILE;
			else if(!g_strcasecmp(optarg, "dir"))
				retval->inputFilter=LSM__INPUT_FILTER__DIR;
			else if(!g_strcasecmp(optarg, "dirFull"))
				retval->inputFilter=LSM__INPUT_FILTER__DIR_FULL;
			else {
				g_critical("Invalid input filter %s, see lsm2 -h for help !", optarg);
				exit(1);
			}
			break;

		case 'o':
			if(!g_strcasecmp(optarg, "xmlFile"))
				retval->outputFilter=LSM__OUTPUT_FILTER__XML_FILE;
			else if(!g_strcasecmp(optarg, "dir"))
				retval->outputFilter=LSM__OUTPUT_FILTER__DIR;
			else if(!g_strcasecmp(optarg, "string"))
				retval->outputFilter=LSM__OUTPUT_FILTER__STRING;
			else if(!g_strcasecmp(optarg, "html"))
				retval->outputFilter=LSM__OUTPUT_FILTER__HTML;
			else {
				g_critical("Invalid output filter %s, see lsm2 -h for help !", optarg);
				exit(1);
			}
			break;
		
		default:
			exit(1);
			break;
		}
	}
	/* processing arguments */

	/* if there is an argument more, it is the directory to analyse */
	if (optind < argc) {
		retval->input = g_strdup(argv[optind]);
		optind++;
		if (optind < argc) {
			retval->output = g_strdup(argv[optind]);
			optind++;
		}
	}
	return retval;
}
static void
LsmOpts_destroy(LsmOpts* self)
{
	if(self->input)
		g_free(self->input);
	if(self->output)
		g_free(self->output);
	g_free(self);
}

int main(int argc, char **argv){
	LsmOpts* lsmOpts;
	LsmTree* lsmTree;
	char *stream;

	lsmOpts=LsmOpts_new(argc, argv);

	switch(lsmOpts->inputFilter) {
	case LSM__INPUT_FILTER__XML_FILE:
		lsmTree=LsmTree_newFromLsmXmlFile(lsmOpts->input);
		break;
	case LSM__INPUT_FILTER__DIR:
		lsmTree=LsmTree_newFromDirectory(lsmOpts->input?lsmOpts->input:".", FALSE);
		break;
	case LSM__INPUT_FILTER__DIR_FULL:
		lsmTree=LsmTree_newFromDirectory(lsmOpts->input?lsmOpts->input:".", TRUE);
		break;
	}
	
	g_return_val_if_fail(lsmTree, 1);
	
	switch(lsmOpts->outputFilter) {
	case LSM__OUTPUT_FILTER__DIR:
		LsmTree_toDirectory(lsmTree, lsmOpts->output?lsmOpts->output:".");
		break;
	case LSM__OUTPUT_FILTER__XML_FILE:
		if (lsmOpts->output)
			LsmTree_toLsmXmlFile(lsmTree, lsmOpts->output);
		else {
	                LsmTree_toLsmXmlStream(lsmTree, &stream);
			printf(stream);
			g_free(stream);
		}
		break;
	case LSM__OUTPUT_FILTER__STRING:
		LsmTree_Export_toString(lsmTree, &stream);
		if(lsmOpts->output) {
			FILE* fd;
			fd=fopen(lsmOpts->output, "w");
			g_return_val_if_fail(fd, 1);
			fprintf(fd, stream);
			fclose(fd);
		}
		else
			printf(stream);
		g_free(stream);
		break;
	case LSM__OUTPUT_FILTER__HTML:
		LsmTree_Export_toHTML(lsmTree, &stream);
		if(lsmOpts->output) {
			FILE* fd;
			fd=fopen(lsmOpts->output, "w");
			g_return_val_if_fail(fd, 1);
			fprintf(fd, stream);
			fclose(fd);
		}
		else
			printf(stream);
		g_free(stream);
		break;
	}

	LsmOpts_destroy(lsmOpts);

	return(0); 
}	
